@@ -625,8 +625,8 @@ void ghashbase::fit(viewport &view, int force) {
    bigint xmax = 1 ;
    bigint ymin = -1 ;
    bigint ymax = 1 ;
-   int xgoal = view.getwidth() - 2 ;
-   int ygoal = view.getheight() - 2 ;
+   int xgoal = view.getwidth() ;
+   int ygoal = view.getheight() ;
    if (xgoal < 8)
       xgoal = 8 ;
    if (ygoal < 8)
@@ -821,17 +821,13 @@ void ghashbase::fit(viewport &view, int force) {
       if (view.contains(xmin, ymin) && view.contains(xmax, ymax))
          return ;
    }
-   xmin += xmax ;
-   xmin >>= 1 ;
-   ymin += ymax ;
-   ymin >>= 1 ;
    int mag = - currdepth - 1 ;
    while (xsize <= xgoal && ysize <= ygoal && mag < MAX_MAG) {
       mag++ ;
       xsize *= 2 ;
       ysize *= 2 ;
    }
-   view.setpositionmag(xmin, ymin, mag) ;
+   view.setpositionmag(xmin, xmax, ymin, ymax, mag) ;
 }
 void ghashbase::lowerRightPixel(bigint &x, bigint &y, int mag) {
    if (mag >= 0)
