@@ -28,7 +28,6 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 const int logbmsize = 8 ;                 // *must* be 8 in this code
 const int bmsize = (1<<logbmsize) ;
-const int byteoff = (bmsize/8) ;
 const int ibufsize = (bmsize*bmsize/32) ;
 static unsigned int ibigbuf[ibufsize] ;   // shared buffer for 256x256 pixels
 static unsigned char *bigbuf = (unsigned char *)ibigbuf ;
@@ -58,7 +57,7 @@ void qlifealgo::renderbm(int x, int y, int xsize, int ysize) {
    }
    ry = uviewh - ry - rh ;
 
-   // AKT: eventually we'll draw directly into pixbuf rather than bigbuf!!!
+   // AKT: eventually we might draw directly into pixbuf rather than bigbuf
    if (pmag > 1) {
       // convert each bigbuf byte into 8 bytes of state data
       int j = 0;
@@ -451,6 +450,10 @@ void qlifealgo::fill_ll(int d) {
    coor.second.mul_smallint(-1) ;
    coor.first -= bmin ;
    coor.second -= bmin ;
+   if (oddgen) {
+      coor.first -= 1 ;
+      coor.second -= 1 ;
+   }
    int bitsreq = coor.first.bitsreq() ;
    int bitsreq2 = coor.second.bitsreq() ;
    if (bitsreq2 > bitsreq)
@@ -598,10 +601,6 @@ void qlifealgo::draw(viewport &viewarg, liferender &renderarg) {
    */
    xoff = -llx ;
    yoff = -lly ;
-   if (mag == 0) {
-      xoff += oddgen ;
-      yoff += oddgen ;
-   }
    int wd = 2 ;
    if (d >= mag)
       wd = 2 << (d-mag) ;
@@ -910,8 +909,8 @@ void qlifealgo::fit(viewport &view, int force) {
    oddgen = getGeneration().odd() ;
    kadd = oddgen ? 8 : 0 ;
    quickb = 0xfff << (8 + oddgen * 12) ;
-   int xgoal = view.getwidth() - 2 ;
-   int ygoal = view.getheight() - 2 ;
+   int xgoal = view.getwidth() ;
+   int ygoal = view.getheight() ;
    if (xgoal < 8)
       xgoal = 8 ;
    if (ygoal < 8)
@@ -1026,10 +1025,6 @@ void qlifealgo::fit(viewport &view, int force) {
       if (view.contains(xmin, ymin) && view.contains(xmax, ymax))
          return ;
    }
-   xmin += xmax ;
-   xmin >>= 1 ;
-   ymin += ymax ;
-   ymin >>= 1 ;
    int mag = - bitval ;
    while (2 * xsize <= xgoal && 2 * ysize <= ygoal && mag < MAX_MAG) {
       mag++ ;
@@ -1041,22 +1036,26 @@ void qlifealgo::fit(viewport &view, int force) {
       xsize /= 2 ;
       ysize /= 2 ;
    }
-   view.setpositionmag(xmin, ymin, mag) ;
+   view.setpositionmag(xmin, xmax, ymin, ymax, mag) ;
 }
 /**
- *   This is not fixed for qlife yet.  FIXME
+ *   Fixed for qlife.
  */
 void qlifealgo::lowerRightPixel(bigint &x, bigint &y, int mag) {
    if (mag >= 0)
      return ;
-   x += bmin ;
+   x -= oddgen ;
+   x -= bmin ;
    x >>= -mag ;
    x <<= -mag ;
-   x -= bmin ;   
+   x += bmin ;   
+   x += oddgen ;
    y -= 1 ;
    y += bmin ;
+   y += oddgen ;
    y >>= -mag ;
    y <<= -mag ;
    y -= bmin ;
    y += 1 ;
+   y -= oddgen ;
 }
