@@ -50,6 +50,10 @@
     #import "PatternViewController.h"   // for UpdateStatus, BeginProgress, etc
 #endif
 
+#ifdef WEB_GUI
+    #include "webcalls.h"		// for UpdateStatus, BeginProgress, etc
+#endif
+
 // -----------------------------------------------------------------------------
 
 bool generating = false;        // currently generating pattern?
@@ -1150,7 +1154,7 @@ void NextGeneration(bool useinc)
 
     // autofit is only used when doing many gens
     if (currlayer->autofit && (generating || useinc)) {
-        //!!! FitInView(0);
+        FitInView(0);
     }
 
     if (draw_pending) {
@@ -1687,6 +1691,11 @@ static void CreateOneRule(const std::string& rulefile, const std::string& folder
         oss << "Could not create rule file:\n" << rulepath.c_str();
         throw std::runtime_error(oss.str().c_str());
     }
+    
+    #ifdef WEB_GUI
+        // ensure the .rule file persists beyond the current session
+        CopyRuleToLocalStorage(rulepath.c_str());
+    #endif
 
     // append created file to htmlinfo
     htmlinfo += "<a href=\"open:";
