@@ -26,6 +26,10 @@
     #include "jnicalls.h"   // for SwitchToPatternTab, ShowTextFile, ShowHelp, etc
 #endif
 
+#ifdef WEB_GUI
+    #include "webcalls.h"   // for SwitchToPatternTab, ShowTextFile, ShowHelp, etc
+#endif
+
 #ifdef IOS_GUI
     #import "GollyAppDelegate.h"        // for SwitchToPatternTab
     #import "HelpViewController.h"      // for ShowHelp
@@ -85,6 +89,21 @@ void SetPatternTitle(const char* filename)
 
 // -----------------------------------------------------------------------------
 
+bool SaveCurrentLayer()
+{
+    if (currlayer->algo->isEmpty()) return true;    // no need to save empty universe
+
+#ifdef WEB_GUI
+    // show a modal dialog that lets user save their changes
+    return WebSaveChanges();
+#else
+    // currently ignored in Android and iOS versions
+    return true;    
+#endif
+}
+
+// -----------------------------------------------------------------------------
+
 void CreateUniverse()
 {
     // save current rule
@@ -108,7 +127,7 @@ void NewPattern(const char* title)
 {
     if (generating) Warning("Bug detected in NewPattern!");
 
-    //!!! if (askonnew && currlayer->dirty && !SaveCurrentLayer()) return;
+    if (currlayer->dirty && asktosave && !SaveCurrentLayer()) return;
 
     currlayer->savestart = false;
     currlayer->currfile.clear();
@@ -159,7 +178,7 @@ bool LoadPattern(const char* path, const char* newtitle)
 
     // newtitle is only empty if called from ResetPattern/RestorePattern
     if (newtitle[0] != 0) {
-        //!!! if (askonload && currlayer->dirty && !SaveCurrentLayer()) return false;
+        if (currlayer->dirty && asktosave && !SaveCurrentLayer()) return false;
 
         currlayer->savestart = false;
         currlayer->currfile = path;
@@ -287,6 +306,10 @@ bool CopyTextToClipboard(const char* text)
     return AndroidCopyTextToClipboard(text);
 #endif
 
+#ifdef WEB_GUI
+    return WebCopyTextToClipboard(text);
+#endif
+
 #ifdef IOS_GUI
     UIPasteboard *pboard = [UIPasteboard generalPasteboard];
     pboard.string = [NSString stringWithCString:text encoding:NSUTF8StringEncoding];
@@ -302,6 +325,10 @@ bool GetTextFromClipboard(std::string& text)
     return AndroidGetTextFromClipboard(text);
 #endif
 
+#ifdef WEB_GUI
+    return WebGetTextFromClipboard(text);
+#endif
+
 #ifdef IOS_GUI
     UIPasteboard *pboard = [UIPasteboard generalPasteboard];
     NSString *str = pboard.string;
@@ -433,11 +460,11 @@ bool ExtractZipEntry(const std::string& zippath, const std::string& entryname, c
                 }
                 if (file_info.uncompressed_size == 0) {
                     Warning("Zip entry is empty!");
-                } else if (bytesRead == file_info.uncompressed_size) {
+                } else if (bytesRead == (long)(file_info.uncompressed_size)) {
                     // write zipdata to outfile
                     FILE* f = fopen(outfile.c_str(), "wb");
                     if (f) {
-                        if (fwrite(zipdata, 1, bytesRead, f) != bytesRead) {
+                        if (fwrite(zipdata, 1, bytesRead, f) != (size_t)bytesRead) {
                             Warning("Could not write data for zip entry!");
                         } else {
                             ok = true;
@@ -553,11 +580,11 @@ static bool RuleInstalled(unzFile zfile, unz_file_info& info, const std::string&
     }
     
     bool ok = true;
-    if (bytesRead == info.uncompressed_size) {
+    if (bytesRead == (long)(info.uncompressed_size)) {
         // write zipdata to outfile
         FILE* f = fopen(outfile.c_str(), "wb");
         if (f) {
-            if (fwrite(zipdata, 1, bytesRead, f) != bytesRead) {
+            if (fwrite(zipdata, 1, bytesRead, f) != (size_t)bytesRead) {
                 Warning("Could not write data for zip entry!");
                 ok = false;
             }
@@ -724,6 +751,10 @@ void OpenZipFile(const char* zippath)
                                         contents += "(overrides file in Rules folder)";
                                     }
                                 }
+                                #ifdef WEB_GUI
+                                    // ensure the .rule file persists beyond the current session
+                                    CopyRuleToLocalStorage(outfile.c_str());
+                                #endif
                             } else {
                                 // file could not be installed
                                 contents += indent;
@@ -966,6 +997,10 @@ bool DownloadFile(const std::string& url, const std::string& filepath)
     return AndroidDownloadFile(url, filepath);
 #endif
 
+#ifdef WEB_GUI
+    return WebDownloadFile(url, filepath);
+#endif
+
 #ifdef IOS_GUI
     NSURL *nsurl = [NSURL URLWithString:[NSString stringWithCString:url.c_str() encoding:NSUTF8StringEncoding]];
     if (nsurl == nil) {
