@@ -41,6 +41,10 @@
     #include "jnicalls.h"   // for BeginProgress, etc
 #endif
 
+#ifdef WEB_GUI
+    #include "webcalls.h"   // for BeginProgress, etc
+#endif
+
 #ifdef IOS_GUI
     #import "PatternViewController.h"   // for BeginProgress, etc
 #endif
@@ -130,6 +134,9 @@ void Selection::DisplaySize()
     msg += Stringify(wd);
     msg += " x ";
     msg += Stringify(ht);
+    #ifdef WEB_GUI
+        msg += " (right-click on it to perform actions)";
+    #endif
     SetMessage(msg.c_str());
 }
 
