@@ -43,6 +43,11 @@
 #ifdef ANDROID_GUI
     #include "jnicalls.h"   // for UpdatePattern, BeginProgress, etc
 #endif
+
+#ifdef WEB_GUI
+    #include "webcalls.h"   // for UpdatePattern, BeginProgress, etc
+#endif
+
 #ifdef IOS_GUI
     #import "PatternViewController.h"   // for UpdatePattern, BeginProgress, etc
 #endif
@@ -150,8 +155,8 @@ void FitInView(int force)
         int pwd, pht;
         int mag = MAX_MAG;
         while (true) {
-            pwd = mag >= 0 ? (pastebox.width << mag) : (pastebox.width >> -mag);
-            pht = mag >= 0 ? (pastebox.height << mag) : (pastebox.height >> -mag);
+            pwd = mag >= 0 ? (pastebox.width << mag) - 1 : (pastebox.width >> -mag);
+            pht = mag >= 0 ? (pastebox.height << mag) - 1 : (pastebox.height >> -mag);
             if (vwd >= pwd && vht >= pht) {
                 // all of paste image can fit within viewport at this mag
                 break;
@@ -163,13 +168,8 @@ void FitInView(int force)
         currlayer->view->setpositionmag(bigint::zero, bigint::zero, mag);
         
         // move paste image to middle of viewport
-        if (mag >= 0) {
-            pastex = (vwd - pwd) / 2 + (1 << mag);
-            pastey = (vht - pht) / 2 + (1 << mag);
-        } else {
-            pastex = (vwd - pwd) / 2;
-            pastey = (vht - pht) / 2;
-        }
+        pastex = (vwd - pwd) / 2;
+        pastey = (vht - pht) / 2;
     } else {
         // fit current pattern in viewport
         // (if no pattern this will set mag to MAX_MAG and move to origin)
@@ -660,6 +660,12 @@ void TouchBegan(int x, int y)
     } else if (currlayer->touchmode == movemode) {
         StartMovingView(x, y);
 
+    } else if (currlayer->touchmode == zoominmode) {
+        ZoomInPos(x, y);
+
+    } else if (currlayer->touchmode == zoomoutmode) {
+        ZoomOutPos(x, y);
+
     }
 }
 
@@ -717,30 +723,6 @@ void TouchEnded()
 
 // -----------------------------------------------------------------------------
 
-int SmallScroll(int xysize)
-{
-    int amount;
-    int mag = currlayer->view->getmag();
-    if (mag > 0) {
-        // scroll an integral number of cells (1 cell = 2^mag pixels)
-        if (mag < 3) {
-            amount = ((xysize >> mag) / 20) << mag;
-            if (amount == 0) amount = 1 << mag;
-            return amount;
-        } else {
-            // grid lines are visible so scroll by only 1 cell
-            return 1 << mag;
-        }
-    } else {
-        // scroll by approx 5% of current wd/ht
-        amount = xysize / 20;
-        if (amount == 0) amount = 1;
-        return amount;
-    }
-}
-
-// -----------------------------------------------------------------------------
-
 bool CopyRect(int itop, int ileft, int ibottom, int iright,
               lifealgo* srcalgo, lifealgo* destalgo,
               bool erasesrc, const char* progmsg)
@@ -1056,6 +1038,11 @@ bool ClipboardContainsRule()
         return true;
     }
     fclose(rulefile);
+    
+    #ifdef WEB_GUI
+        // ensure the .rule file persists beyond the current session
+        CopyRuleToLocalStorage(rulepath.c_str());
+    #endif
 
     // now switch to the newly created rule
     ChangeRule(rulename);
@@ -1090,7 +1077,12 @@ void PasteClipboard()
         if ( OutsideLimits(top, left, bottom, right) ) {
             ErrorMessage("Clipboard pattern is too big.");
         } else {
-            DisplayMessage("Drag paste image to desired location then tap Paste button.");
+            #ifdef WEB_GUI
+                DisplayMessage("Drag paste image to desired location then right-click on it.");
+            #else
+                // Android and iOS devices use a touch screen
+                DisplayMessage("Drag paste image to desired location then tap Paste button.");
+            #endif
             waitingforpaste = true;
 
             // set initial position of pasterect's top left corner to near top left corner
@@ -1472,10 +1464,10 @@ bool RotatePastePattern(bool clockwise)
 void ZoomInPos(int x, int y)
 {
     // zoom in to given point
-    TestAutoFit();
     if (currlayer->view->getmag() < MAX_MAG) {
+        TestAutoFit();
         currlayer->view->zoom(x, y);
-        UpdatePatternAndStatus();
+        UpdateEverything();
     } else {
         Beep();   // can't zoom in any further
     }
@@ -1488,5 +1480,132 @@ void ZoomOutPos(int x, int y)
     // zoom out from given point
     TestAutoFit();
     currlayer->view->unzoom(x, y);
-    UpdatePatternAndStatus();
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanUp(int amount)
+{
+    TestAutoFit();
+    currlayer->view->move(0, -amount);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanDown(int amount)
+{
+    TestAutoFit();
+    currlayer->view->move(0, amount);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanLeft(int amount)
+{
+    TestAutoFit();
+    currlayer->view->move(-amount, 0);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanRight(int amount)
+{
+    TestAutoFit();
+    currlayer->view->move(amount, 0);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanNE()
+{
+    TestAutoFit();
+    int xamount = SmallScroll(currlayer->view->getwidth());
+    int yamount = SmallScroll(currlayer->view->getheight());
+    int amount = (xamount < yamount) ? xamount : yamount;
+    currlayer->view->move(amount, -amount);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanNW()
+{
+    TestAutoFit();
+    int xamount = SmallScroll(currlayer->view->getwidth());
+    int yamount = SmallScroll(currlayer->view->getheight());
+    int amount = (xamount < yamount) ? xamount : yamount;
+    currlayer->view->move(-amount, -amount);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanSE()
+{
+    TestAutoFit();
+    int xamount = SmallScroll(currlayer->view->getwidth());
+    int yamount = SmallScroll(currlayer->view->getheight());
+    int amount = (xamount < yamount) ? xamount : yamount;
+    currlayer->view->move(amount, amount);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+void PanSW()
+{
+    TestAutoFit();
+    int xamount = SmallScroll(currlayer->view->getwidth());
+    int yamount = SmallScroll(currlayer->view->getheight());
+    int amount = (xamount < yamount) ? xamount : yamount;
+    currlayer->view->move(-amount, amount);
+    UpdateEverything();
+}
+
+// -----------------------------------------------------------------------------
+
+int SmallScroll(int xysize)
+{
+    int amount;
+    int mag = currlayer->view->getmag();
+    if (mag > 0) {
+        // scroll an integral number of cells (1 cell = 2^mag pixels)
+        if (mag < 3) {
+            amount = ((xysize >> mag) / 20) << mag;
+            if (amount == 0) amount = 1 << mag;
+            return amount;
+        } else {
+            // grid lines are visible so scroll by only 1 cell
+            return 1 << mag;
+        }
+    } else {
+        // scroll by approx 5% of current wd/ht
+        amount = xysize / 20;
+        if (amount == 0) amount = 1;
+        return amount;
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+int BigScroll(int xysize)
+{
+    int amount;
+    int mag = currlayer->view->getmag();
+    if (mag > 0) {
+        // scroll an integral number of cells (1 cell = 2^mag pixels)
+        amount = ((xysize >> mag) * 9 / 10) << mag;
+        if (amount == 0) amount = 1 << mag;
+        return amount;
+    } else {
+        // scroll by approx 90% of current wd/ht
+        amount = xysize * 9 / 10;
+        if (amount == 0) amount = 1;
+        return amount;
+    }
 }
