@@ -41,7 +41,7 @@
 #import <OpenGLES/ES1/gl.h>
 #import <OpenGLES/ES1/glext.h>
 
-#import "PatternViewController.h"   // for PauseGenerating, ResumeGenerating
+#import "PatternViewController.h"   // for PauseGenerating, ResumeGenerating, StopIfGenerating
 #import "PatternView.h"
 
 @implementation PatternView
@@ -89,7 +89,11 @@ static GLuint viewFramebuffer = 0;
         // we only do 2D drawing
         glDisable(GL_DEPTH_TEST);
         glDisable(GL_DITHER);
-        glDisable(GL_TEXTURE_2D);
+        glDisable(GL_MULTISAMPLE);
+        glDisable(GL_STENCIL_TEST);
+        glDisable(GL_FOG);
+        
+        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
         glEnableClientState(GL_VERTEX_ARRAY);
         
         glEnable(GL_BLEND);
@@ -239,7 +243,7 @@ static GLuint viewFramebuffer = 0;
 
     [EAGLContext setCurrentContext:context];
 
-    DrawPattern(context, tileindex);
+    DrawPattern(tileindex);
     
     // display the buffer
     glBindRenderbufferOES(GL_RENDERBUFFER_OES, viewRenderbuffer);
@@ -402,6 +406,7 @@ static UIActionSheet *selsheet;
         @"Clear",
         @"Clear Outside",
         @"Shrink",
+        @"Fit",
         [NSString stringWithFormat:@"Random Fill (%d%%)", randomfill],
         @"Flip Top-Bottom",
         @"Flip Left-Right",
@@ -447,9 +452,9 @@ static int globalButton;
 - (void)doDelayedAction
 {
     if (globalSheet == selsheet) {
-        if (generating && globalButton >= 1 && globalButton <= 12 &&
-            globalButton != 2 && globalButton != 5) {
-            // temporarily stop generating for all actions except Remove, Copy and Shrink
+        if (generating && globalButton >= 1 && globalButton <= 13 &&
+            globalButton != 2 && globalButton != 5 && globalButton != 6) {
+            // temporarily stop generating for all actions except Remove, Copy, Shrink, Fit
             PauseGenerating();
             if (event_checker > 0) {
                 // try again after a short delay that gives time for NextGeneration() to terminate
@@ -464,13 +469,14 @@ static int globalButton;
             case 3:  ClearSelection(); break;
             case 4:  ClearOutsideSelection(); break;
             case 5:  ShrinkSelection(false); break;                 // WARNING: above test assumes Shrink is index 5
-            case 6:  RandomFill(); break;
-            case 7:  FlipSelection(true); break;
-            case 8:  FlipSelection(false); break;
-            case 9:  RotateSelection(true); break;
-            case 10: RotateSelection(false); break;
-            case 11: currlayer->currsel.Advance(); break;
-            case 12: currlayer->currsel.AdvanceOutside(); break;    // WARNING: above test assumes 12 is last index
+            case 6:  FitSelection(); break;                         // WARNING: above test assumes Fit is index 6
+            case 7:  RandomFill(); break;
+            case 8:  FlipSelection(true); break;
+            case 9:  FlipSelection(false); break;
+            case 10: RotateSelection(true); break;
+            case 11: RotateSelection(false); break;
+            case 12: currlayer->currsel.Advance(); break;
+            case 13: currlayer->currsel.AdvanceOutside(); break;    // WARNING: above test assumes 13 is last index
             default: break;
         }
         ResumeGenerating();
@@ -516,7 +522,10 @@ static double prevtime = 0.0;   // used to detect a double tap
         if (TimeInSeconds() - prevtime < 0.3) {
             // double tap
             if (waitingforpaste && PointInPasteImage(pt.x, pt.y)) {
-                // display action sheet that lets user abort/paste/flip/rotate/etc
+                // if generating then stop (consistent with doPaste in PatternViewController.m)
+                StopIfGenerating();
+                ClearMessage();
+                // now display action sheet that lets user abort/paste/flip/rotate/etc
                 [self doPasteAction];
             
             } else if (currlayer->touchmode == selectmode && SelectionExists() && PointInSelection(pt.x, pt.y)) {
@@ -541,30 +550,4 @@ static double prevtime = 0.0;   // used to detect a double tap
 
 // -----------------------------------------------------------------------------
 
-/* too many problems:
-
-- (void)doubleTap:(UITapGestureRecognizer *)gestureRecognizer
-{
-    if ([gestureRecognizer state] == UIGestureRecognizerStateEnded) {
-        CGPoint pt = [gestureRecognizer locationInView:self];
-        ClearMessage();
-        if (waitingforpaste && PointInPasteImage(pt.x, pt.y)) {
-            // display action sheet that lets user abort/paste/flip/rotate/etc
-            [self doPasteAction];
-        } else if (SelectionExists() && PointInSelection(pt.x, pt.y)) {
-            // display action sheet that lets user remove/cut/copy/clear/etc
-            [self doSelectionAction];
-        } else if (currlayer->touchmode == drawmode) {
-            // best to do the drawing, but there's a nasty problem: we only get
-            // location of 1st tap, even if 2nd tap was in a different location
-            TouchBegan(pt.x, pt.y);
-            TouchEnded();
-        }
-    }
-}
-
-*/
-
-// -----------------------------------------------------------------------------
-
 @end
