@@ -25,9 +25,9 @@
 #include "viewport.h"    // for MAX_MAG
 
 #include "utils.h"       // for Warning, event_checker
-#include "status.h"      // for DisplayMessage
+#include "status.h"      // for SetMessage, DisplayMessage
 #include "algos.h"       // for InitAlgorithms, algoinfo
-#include "prefs.h"       // for GetPrefs, SavePrefs
+#include "prefs.h"       // for GetPrefs, SavePrefs, userdir, etc
 #include "layer.h"       // for AddLayer, currlayer
 #include "file.h"        // for NewPattern
 #include "control.h"     // for generating, StartGenerating, etc
@@ -89,6 +89,76 @@ static int pausecount = 0;              // if > 0 then genTimer needs to be rest
 
 // -----------------------------------------------------------------------------
 
+static void CreateDocSubdir(NSString *subdirname)
+{
+    // create given subdirectory inside Documents
+    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
+    NSString *path = [[paths objectAtIndex:0] stringByAppendingPathComponent:subdirname];
+    if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
+        // do nothing if subdir already exists
+    } else {
+        NSError *error;
+        if (![[NSFileManager defaultManager] createDirectoryAtPath:path
+                                       withIntermediateDirectories:NO
+                                                        attributes:nil
+                                                             error:&error]) {
+            NSLog(@"Error creating %@ subdirectory: %@", subdirname, error);
+        }
+    }
+}
+
+// -----------------------------------------------------------------------------
+
+static void InitPaths()
+{
+    // init userdir to directory containing Golly.app
+    userdir = [NSHomeDirectory() cStringUsingEncoding:NSUTF8StringEncoding];
+    userdir += "/";
+
+    // init savedir to userdir/Documents/Saved/
+    savedir = userdir + "Documents/Saved/";
+    CreateDocSubdir(@"Saved");
+
+    // init downloaddir to userdir/Documents/Downloads/
+    downloaddir = userdir + "Documents/Downloads/";
+    CreateDocSubdir(@"Downloads");
+
+    // init userrules to userdir/Documents/Rules/
+    userrules = userdir + "Documents/Rules/";
+    CreateDocSubdir(@"Rules");
+
+    // supplied patterns, rules, help are bundled inside Golly.app
+    supplieddir = userdir + "Golly.app/";
+    patternsdir = supplieddir + "Patterns/";
+    rulesdir = supplieddir + "Rules/";
+    helpdir = supplieddir + "Help/";
+
+    // init tempdir to userdir/tmp/
+    tempdir = userdir + "tmp/";
+
+    // init path to file that stores clipboard data
+    clipfile = tempdir + "golly_clipboard";
+    
+    // init path to file that stores user preferences
+    prefsfile = userdir + "Library/Preferences/GollyPrefs";
+
+    /*
+    NSLog(@"userdir =     %s", userdir.c_str());
+    NSLog(@"savedir =     %s", savedir.c_str());
+    NSLog(@"downloaddir = %s", downloaddir.c_str());
+    NSLog(@"userrules =   %s", userrules.c_str());
+    NSLog(@"supplieddir = %s", supplieddir.c_str());
+    NSLog(@"patternsdir = %s", patternsdir.c_str());
+    NSLog(@"rulesdir =    %s", rulesdir.c_str());
+    NSLog(@"helpdir =     %s", helpdir.c_str());
+    NSLog(@"tempdir =     %s", tempdir.c_str());
+    NSLog(@"clipfile =    %s", clipfile.c_str());
+    NSLog(@"prefsfile =   %s", prefsfile.c_str());
+    */
+}
+
+// -----------------------------------------------------------------------------
+
 - (void)viewDidLoad
 {
     [super viewDidLoad];
@@ -106,8 +176,10 @@ static int pausecount = 0;              // if > 0 then genTimer needs to be rest
     static bool firstload = true;
     if (firstload) {
         firstload = false;          // only do the following once
+        SetMessage("This is Golly 1.2 for iOS.  Copyright 2013 The Golly Gang.");
         MAX_MAG = 5;                // maximum cell size = 32x32
         InitAlgorithms();           // must initialize algoinfo first
+        InitPaths();                // init userdir, etc (must be before GetPrefs)
         GetPrefs();                 // load user's preferences
         SetMinimumStepExponent();   // for slowest speed
         AddLayer();                 // create initial layer (sets currlayer)
@@ -197,11 +269,6 @@ static int pausecount = 0;              // if > 0 then genTimer needs to be rest
 {
 	[super viewDidAppear:animated];
     
-    // not sure why, but we need to update pattView here rather than in viewWillAppear
-    // if viewWillDisappear called stopIfGenerating and event_chacker  was > 0
-    // (presumably something to do with the switch to OpenGL ES; ie. we now call
-    // [pattView refreshPattern] rather than [pattView setNeedsDisplay])
-    
     UpdateEverything();     // calls [pattView refreshPattern]
     
     // restart genTimer if PauseGenTimer() was called earlier
@@ -513,7 +580,7 @@ static int pausecount = 0;              // if > 0 then genTimer needs to be rest
 - (IBAction)doFit:(id)sender
 {
     ClearMessage();
-    currlayer->algo->fit(*currlayer->view, 1);
+    FitInView(1);
 
     [statView setNeedsDisplay];
     [pattView refreshPattern];
@@ -652,6 +719,7 @@ static int pausecount = 0;              // if > 0 then genTimer needs to be rest
 
 - (IBAction)doAction:(id)sender
 {
+    ClearMessage();
     [pattView doSelectionAction];
 }
 
@@ -862,6 +930,13 @@ void ResumeGenerating()
 
 // -----------------------------------------------------------------------------
 
+void StopIfGenerating()
+{
+    [globalController stopIfGenerating];
+}
+
+// -----------------------------------------------------------------------------
+
 void BeginProgress(const char* title)
 {
     if (progresscount == 0) {
