@@ -244,11 +244,11 @@ static void CreateRuleLinks(std::string& htmldata, const std::string& dir,
 {
     NSString *algoname = [NSString stringWithCString:GetAlgoName(algoindex) encoding:NSUTF8StringEncoding];
     if ([algoname isEqualToString:@"RuleLoader"]) {
-        // first check for any .rule/tree/table/colors/icons files in Documents
+        // first check for any .rule/tree/table files in Documents
         // (created by iTunes file sharing) and move them into Documents/Rules/
         MoveSharedFiles();
         
-        // now convert any .tree/table/colors/icons files to .rule files
+        // now convert any .tree/table files to .rule files
         ConvertOldRules();
         
         // create html data with links to the user's .rule files and Golly's supplied .rule files
@@ -282,10 +282,19 @@ static void CreateRuleLinks(std::string& htmldata, const std::string& dir,
 
 // -----------------------------------------------------------------------------
 
+static bool keepalgoindex = false;
+
 - (void)viewWillAppear:(BOOL)animated
 {
     [super viewWillAppear:animated];
 
+    if (keepalgoindex) {
+        // ShowTextFile has finished displaying a modal dialog so we don't want to change algoindex
+        keepalgoindex = false;
+        [self showAlgoHelp];        // user might have created a new .rule file
+        return;
+    }
+
     // save current location
     curroffset[algoindex] = htmlView.scrollView.contentOffset;
     
@@ -634,7 +643,7 @@ static int globalButton;
             std::string question = "Do you really want to delete " + path + "?";
             if (YesNo(question.c_str())) {
                 // delete specified file
-                path = gollydir + path;
+                path = userdir + path;
                 RemoveFile(path);
                 // save current location
                 curroffset[algoindex] = htmlView.scrollView.contentOffset;
@@ -649,14 +658,16 @@ static int globalButton;
             std::string fullpath = path;
             if (path[0] != '/') {
                 if (fullpath.find("Patterns/") == 0 || fullpath.find("Rules/") == 0) {
-                    // Patterns and Rules directories are inside app bundle so prepend gollydir/Golly.app/
-                    fullpath = gollydir + "Golly.app/" + fullpath;
+                    // Patterns and Rules directories are inside supplieddir
+                    fullpath = supplieddir + fullpath;
                 } else {
-                    fullpath = gollydir + fullpath;
+                    fullpath = userdir + fullpath;
                 }
             }
             // we pass self to ShowTextFile so it doesn't use current tab's view controller
             ShowTextFile(fullpath.c_str(), self);
+            // tell viewWillAppear not to reset algoindex to currlayer->algtype
+            keepalgoindex = true;
             return NO;
         }
     }
