@@ -7,7 +7,7 @@ echo 'gollydatadir = $(pkgdatadir)/Patterns/Life/' >>sources.am
 echo nobase_dist_gollydata_DATA = `find ../../{Help,Patterns,Rules,Scripts} -type f | sort` >>sources.am
 echo EXTRA_DIST = \
 	../../gui-wx/{makefile-{gtk,mac,win},CMakeLists.txt,Info.plist.in,*.mk,*.rc,configure/autogen.sh,icons,bitmaps} \
-	../../gui-common $(find ../../gui-{android,ios} -type f | sort) \
+	../../gui-common $(find ../../gui-{android,ios,web} -type f | sort) \
 	../../docs >>sources.am
 
 aclocal -I m4
