@@ -65,9 +65,6 @@ enum {
     STATE_BAR
 };
 
-#ifdef __WXMSW__
-// bitmaps are loaded via .rc file
-#else
 // bitmaps for edit bar buttons
 #include "bitmaps/undo.xpm"
 #include "bitmaps/redo.xpm"
@@ -86,15 +83,17 @@ enum {
 #include "bitmaps/zoomin_down.xpm"
 #include "bitmaps/zoomout_down.xpm"
 #include "bitmaps/allstates_down.xpm"
-#endif
 
 // width and height of bitmap buttons
-#if defined(__WXOSX_COCOA__) || defined(__WXGTK__)
-const int BUTTON_WD = 28;
-const int BUTTON_HT = 28;
+#if defined(__WXOSX_COCOA__) && wxCHECK_VERSION(3,0,0)
+    const int BUTTON_WD = 24;
+    const int BUTTON_HT = 24;
+#elif defined(__WXOSX_COCOA__) || defined(__WXGTK__)
+    const int BUTTON_WD = 28;
+    const int BUTTON_HT = 28;
 #else
-const int BUTTON_WD = 24;
-const int BUTTON_HT = 24;
+    const int BUTTON_WD = 24;
+    const int BUTTON_HT = 24;
 #endif
 
 // -----------------------------------------------------------------------------
@@ -218,24 +217,24 @@ EditBar::EditBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, int ht)
 #endif
     
     // init bitmaps for normal state
-    normbutt[UNDO_BUTT] =      wxBITMAP(undo);
-    normbutt[REDO_BUTT] =      wxBITMAP(redo);
-    normbutt[DRAW_BUTT] =      wxBITMAP(draw);
-    normbutt[PICK_BUTT] =      wxBITMAP(pick);
-    normbutt[SELECT_BUTT] =    wxBITMAP(select);
-    normbutt[MOVE_BUTT] =      wxBITMAP(move);
-    normbutt[ZOOMIN_BUTT] =    wxBITMAP(zoomin);
-    normbutt[ZOOMOUT_BUTT] =   wxBITMAP(zoomout);
-    normbutt[ALLSTATES_BUTT] = wxBITMAP(allstates);
+    normbutt[UNDO_BUTT] =      XPM_BITMAP(undo);
+    normbutt[REDO_BUTT] =      XPM_BITMAP(redo);
+    normbutt[DRAW_BUTT] =      XPM_BITMAP(draw);
+    normbutt[PICK_BUTT] =      XPM_BITMAP(pick);
+    normbutt[SELECT_BUTT] =    XPM_BITMAP(select);
+    normbutt[MOVE_BUTT] =      XPM_BITMAP(move);
+    normbutt[ZOOMIN_BUTT] =    XPM_BITMAP(zoomin);
+    normbutt[ZOOMOUT_BUTT] =   XPM_BITMAP(zoomout);
+    normbutt[ALLSTATES_BUTT] = XPM_BITMAP(allstates);
     
     // toggle buttons also have a down state
-    downbutt[DRAW_BUTT] =      wxBITMAP(draw_down);
-    downbutt[PICK_BUTT] =      wxBITMAP(pick_down);
-    downbutt[SELECT_BUTT] =    wxBITMAP(select_down);
-    downbutt[MOVE_BUTT] =      wxBITMAP(move_down);
-    downbutt[ZOOMIN_BUTT] =    wxBITMAP(zoomin_down);
-    downbutt[ZOOMOUT_BUTT] =   wxBITMAP(zoomout_down);
-    downbutt[ALLSTATES_BUTT] = wxBITMAP(allstates_down);
+    downbutt[DRAW_BUTT] =      XPM_BITMAP(draw_down);
+    downbutt[PICK_BUTT] =      XPM_BITMAP(pick_down);
+    downbutt[SELECT_BUTT] =    XPM_BITMAP(select_down);
+    downbutt[MOVE_BUTT] =      XPM_BITMAP(move_down);
+    downbutt[ZOOMIN_BUTT] =    XPM_BITMAP(zoomin_down);
+    downbutt[ZOOMOUT_BUTT] =   XPM_BITMAP(zoomout_down);
+    downbutt[ALLSTATES_BUTT] = XPM_BITMAP(allstates_down);
     
 #ifdef __WXMSW__
     for (int i = 0; i < NUM_BUTTONS; i++) {
@@ -759,7 +758,12 @@ void EditBar::OnButtonUp(wxMouseEvent& event)
 void EditBar::AddButton(int id, const wxString& tip)
 {
     ebbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos),
-                                    wxSize(BUTTON_WD, BUTTON_HT));
+#if defined(__WXOSX_COCOA__) && wxCHECK_VERSION(3,0,0)
+                                    wxSize(BUTTON_WD, BUTTON_HT), wxBORDER_SIMPLE
+#else
+                                    wxSize(BUTTON_WD, BUTTON_HT)
+#endif
+                                    );
     if (ebbutt[id] == NULL) {
         Fatal(_("Failed to create edit bar button!"));
     } else {
