@@ -79,9 +79,9 @@ wxString GetRuleName(const wxString& rulestring)
     // first look for given rulestring in namedrules; if user has created a name like
     // "Life on torus" for "B3/S23:T100,200" then this will find that name
     for (size_t i = 0; i < namedrules.GetCount(); i++) {
-        wxString thisrule = namedrules[i].AfterFirst('|');       // rule is after '|'
+        wxString thisrule = namedrules[i].AfterFirst('|');          // rule is after '|'
         if ( rulestring.IsSameAs(thisrule,false) ) {
-            rulename = namedrules[i].BeforeFirst('|');            // name is before '|'
+            rulename = namedrules[i].BeforeFirst('|');              // name is before '|'
             if ( rulesuffix.Length() > 0 ) {
                 // still append suffix so user sees "Life on torus:T100,200"
                 rulename += rulesuffix;
@@ -94,9 +94,9 @@ wxString GetRuleName(const wxString& rulestring)
         // look for ruleprefix in namedrules; if there is no explicit name for
         // "B3/S23:T100,200" then this will find "Life" and user will see "Life:T100,200"
         for (size_t i = 0; i < namedrules.GetCount(); i++) {
-            wxString thisrule = namedrules[i].AfterFirst('|');    // rule is after '|'
+            wxString thisrule = namedrules[i].AfterFirst('|');      // rule is after '|'
             if ( ruleprefix.IsSameAs(thisrule,false) ) {
-                rulename = namedrules[i].BeforeFirst('|');         // name is before '|'
+                rulename = namedrules[i].BeforeFirst('|');          // name is before '|'
                 rulename += rulesuffix;
                 return rulename;
             }
@@ -112,8 +112,8 @@ wxString GetRuleName(const wxString& rulestring)
 
 // globals used in AlgoHelp and RuleDialog classes:
 
-static wxTextCtrl* ruletext;        // text box for user to type in rule
-static wxStaticText* statusline;    // status line at bottom of dialog
+static wxTextCtrl* ruletext = NULL;        // text box for user to type in rule
+static wxStaticText* statusline = NULL;    // status line at bottom of dialog
 
 // -----------------------------------------------------------------------------
 
@@ -480,13 +480,14 @@ END_EVENT_TABLE()
 
 // -----------------------------------------------------------------------------
 
-RuleDialog::RuleDialog(wxWindow* parent)
+RuleDialog::RuleDialog(wxWindow* parent) 
+    :  htmlwin( NULL )
 {
+    expanded = false;    // tested in RuleDialog::OnSize (called by Create)
+    
     Create(parent, wxID_ANY, _("Set Rule"), wxPoint(rulex,ruley), wxDefaultSize,
            wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
     
-    expanded = false;    // tested in RuleDialog::OnSize
-    
     htmlwin = new AlgoHelp(this, wxID_ANY,
                            // specify small size to avoid clipping scroll bar on resize
                            wxDefaultPosition, wxSize(30,30),
@@ -990,9 +991,13 @@ void RuleDialog::OnSize(wxSizeEvent& event)
         ruleexwd = r.width - minrect.width;
         ruleexht = r.height - minrect.height;
         wxRect htmlrect(minrect.width, 10, wd - minrect.width - 10, ht - 30);
-        htmlwin->SetSize(htmlrect);
+        if( htmlwin ) { 
+            htmlwin->SetSize(htmlrect); 
+        }
         // position status line under bottom left corner
-        statusline->Move(htmlrect.GetLeft(), htmlrect.GetBottom() + 4);
+        if( statusline ) {
+            statusline->Move(htmlrect.GetLeft(), htmlrect.GetBottom() + 4);
+        }
     }
     event.Skip();
 }
