@@ -213,7 +213,12 @@ const char* GSF_getdir(char* dirname)
     
     // need to be careful converting Unicode wxString to char*
     static wxCharBuffer dirbuff;
-    dirbuff = dirpath.mb_str(wxConvLocal);
+    #ifdef __WXMAC__
+        // we need to convert dirpath to decomposed UTF8 so fopen will work
+        dirbuff = dirpath.fn_str();
+    #else
+        dirbuff = dirpath.mb_str(wxConvLocal);
+    #endif
     return (const char*) dirbuff;
 }
 
@@ -607,7 +612,7 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
         if (*oldval != newval) {
             currlayer->drawingstate = newval;
             if (autoupdate) {
-                UpdateEditBar(mainptr->IsActive());
+                UpdateEditBar();
                 updateedit = false;
             } else {
                 // update edit bar in next GSF_update call
@@ -819,7 +824,7 @@ bool GSF_setoption(char* optname, int newval, int* oldval)
     }
     
     if (*oldval != newval) {
-        mainptr->UpdateMenuItems(mainptr->IsActive());
+        mainptr->UpdateMenuItems();
     }
     
     return true;
@@ -1218,7 +1223,7 @@ void GSF_update()
     }
     
     if (updateedit) {
-        UpdateEditBar(mainptr->IsActive());
+        UpdateEditBar();
         updateedit = false;
     }
     
@@ -1385,7 +1390,7 @@ void RunScript(const wxString& filename)
         
         inscript = true;
         
-        mainptr->UpdateUserInterface(mainptr->IsActive());
+        mainptr->UpdateUserInterface();
         
         // temporarily remove accelerators from all menu items
         // so keyboard shortcuts can be passed to script
