@@ -45,17 +45,13 @@
 #include "wxlayer.h"       // for currlayer
 #include "wxtimeline.h"
 
-#ifdef __WXMSW__
-    // bitmaps are loaded via .rc file
-#else
-    // bitmaps for timeline bar buttons
-    #include "bitmaps/record.xpm"
-    #include "bitmaps/stop.xpm"
-    #include "bitmaps/backwards.xpm"
-    #include "bitmaps/forwards.xpm"
-    #include "bitmaps/stopplay.xpm"
-    #include "bitmaps/deltime.xpm"
-#endif
+// bitmaps for timeline bar buttons
+#include "bitmaps/record.xpm"
+#include "bitmaps/stop.xpm"
+#include "bitmaps/backwards.xpm"
+#include "bitmaps/forwards.xpm"
+#include "bitmaps/stopplay.xpm"
+#include "bitmaps/deltime.xpm"
 
 // -----------------------------------------------------------------------------
 
@@ -157,7 +153,10 @@ const int MINSPEED = -10;              // minimum autoplay speed
 const int MAXSPEED = 10;               // maximum autoplay speed
 
 // width and height of bitmap buttons
-#if defined(__WXOSX_COCOA__) || defined(__WXGTK__)
+#if defined(__WXOSX_COCOA__) && wxCHECK_VERSION(3,0,0)
+    const int BUTTON_WD = 24;
+    const int BUTTON_HT = 24;
+#elif defined(__WXOSX_COCOA__) || defined(__WXGTK__)
     const int BUTTON_WD = 28;
     const int BUTTON_HT = 28;
 #else
@@ -180,12 +179,12 @@ TimelineBar::TimelineBar(wxWindow* parent, wxCoord xorg, wxCoord yorg, int wd, i
 #endif
     
     // init bitmaps for normal state
-    normbutt[RECORD_BUTT] =    wxBITMAP(record);
-    normbutt[STOPREC_BUTT] =   wxBITMAP(stop);
-    normbutt[BACKWARDS_BUTT] = wxBITMAP(backwards);
-    normbutt[FORWARDS_BUTT] =  wxBITMAP(forwards);
-    normbutt[STOPPLAY_BUTT] =  wxBITMAP(stopplay);
-    normbutt[DELETE_BUTT] =    wxBITMAP(deltime);
+    normbutt[RECORD_BUTT] =    XPM_BITMAP(record);
+    normbutt[STOPREC_BUTT] =   XPM_BITMAP(stop);
+    normbutt[BACKWARDS_BUTT] = XPM_BITMAP(backwards);
+    normbutt[FORWARDS_BUTT] =  XPM_BITMAP(forwards);
+    normbutt[STOPPLAY_BUTT] =  XPM_BITMAP(stopplay);
+    normbutt[DELETE_BUTT] =    XPM_BITMAP(deltime);
     
 #ifdef __WXMSW__
     for (int i = 0; i < NUM_BUTTONS; i++) {
@@ -613,7 +612,12 @@ void TimelineBar::OnButtonUp(wxMouseEvent& event)
 void TimelineBar::AddButton(int id, const wxString& tip)
 {
     tlbutt[id] = new wxBitmapButton(this, id, normbutt[id], wxPoint(xpos,ypos),
-                                    wxSize(BUTTON_WD, BUTTON_HT));
+#if defined(__WXOSX_COCOA__) && wxCHECK_VERSION(3,0,0)
+                                    wxSize(BUTTON_WD, BUTTON_HT), wxBORDER_SIMPLE
+#else
+                                    wxSize(BUTTON_WD, BUTTON_HT)
+#endif
+                                    );
     if (tlbutt[id] == NULL) {
         Fatal(_("Failed to create timeline bar button!"));
     } else {
