@@ -326,7 +326,7 @@ bool ChangeNode::DoChange(bool undo)
                 }
             }
             // Reset item may become enabled/disabled
-            mainptr->UpdateMenuItems(mainptr->IsActive());
+            mainptr->UpdateMenuItems();
             break;
             
         case namechange:
@@ -1238,8 +1238,8 @@ bool UndoRedo::CanUndo()
     if (allowundo && mainptr->generating) return true;
     
     return !undolist.IsEmpty() && !inscript &&
-    !viewptr->waitingforclick && !viewptr->drawingcells &&
-    !viewptr->selectingcells;
+           !viewptr->waitingforclick && !viewptr->drawingcells &&
+           !viewptr->selectingcells;
 }
 
 // -----------------------------------------------------------------------------
@@ -1247,8 +1247,8 @@ bool UndoRedo::CanUndo()
 bool UndoRedo::CanRedo()
 {
     return !redolist.IsEmpty() && !inscript && !mainptr->generating &&
-    !viewptr->waitingforclick && !viewptr->drawingcells &&
-    !viewptr->selectingcells;
+           !viewptr->waitingforclick && !viewptr->drawingcells &&
+           !viewptr->selectingcells;
 }
 
 // -----------------------------------------------------------------------------
@@ -1265,6 +1265,9 @@ void UndoRedo::UndoChange()
         return;
     }
     
+    // prevent re-entrancy if DoChange calls checkevents
+    if (insideYield) return;
+    
     // get change info from head of undo list and do the change
     wxList::compatibility_iterator node = undolist.GetFirst();
     ChangeNode* change = (ChangeNode*) node->GetData();
@@ -1332,6 +1335,9 @@ void UndoRedo::RedoChange()
     }
     */
     
+    // prevent re-entrancy if DoChange calls checkevents
+    if (insideYield) return;
+    
     // get change info from head of redo list and do the change
     wxList::compatibility_iterator node = redolist.GetFirst();
     ChangeNode* change = (ChangeNode*) node->GetData();
